
/* ************************************************************************
*  File: utils.c                                              Part of ACT *
*  Usage: various internal functions of a utility nature                  *
*                                                                         *
*  All rights reserved.  See license.doc for complete information.        *
*                                                                         *
*  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
*  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
************************************************************************ */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "act.h"

char buf[MAX_STRING_LENGTH], buf1[MAX_STRING_LENGTH], buf2[MAX_STRING_LENGTH];


/*
 * get_line reads the next non-blank line off of the input stream.
 * The newline character is removed from the input.  Lines which begin
 * with '*' are considered to be comments.
 *
 * Returns the number of lines advanced in the file.
 */
int get_line(FILE * fl, char *buf, FILE *actfile)
{
  char temp[256];
  int lines = 0;

  *temp = 0;
  do {
    if (*temp)
      fprintf(actfile, "%s\n", temp);
    lines++;
    fgets(temp, 256, fl);
    if (*temp)
      temp[strlen(temp) - 1] = '\0';
  } while (!feof(fl) && (*temp == '*' || !*temp));

  if (feof(fl))
    return 0;
  else {
    strcpy(buf, temp);
    return lines;
  }
}


/* read and allocate space for a '~'-terminated string from a given file */
void fread_string(FILE * fl, char *error, FILE * outfile)
{
  char buf[MAX_STRING_LENGTH], tmp[512];
  register char *point;
  int done = 0, length = 0, templength = 0;

  *buf = '\0';

  do {
    if (!fgets(tmp, 512, fl)) {
      fprintf(stderr, "SYSERR: fread_string: format error at or near %s\n",
              error);
      exit(1);
    }
    /* If there is a '~', end the string; else an "\r\n" over the '\n'. */
    fputs(tmp, outfile);
    if ((point = strchr(tmp, '~')) != NULL)
      done = 1;

    templength = strlen(tmp);

    if (length + templength >= MAX_STRING_LENGTH) {
      fprintf(stderr, "SYSERR: fread_string: string too large (utils.c)");
      exit(1);
    }
  } while (!done);
}


FILE *get_outputfile(char *fn, int mode)
{
  char name[256];
  FILE *temp;

  if (mode == MODE_ACT)
    sprintf(name, "%s.act", fn);
  else if (mode == MODE_UNACT) {
    strcpy(name, fn);
    while((*(name + strlen(name) - 1) != '.') && *name)
      *(name + strlen(name) - 1) = 0;
    if (!(*name) || !(*(name + 1)))
      sprintf(name, "%s.unact", fn);
    else
      *(name + strlen(name) - 1) = 0;
  } else {
    fprintf(stderr, "Illegal mode of %d in get_outputfile()\n", mode);
    exit(1);
  }
  
  if ((temp = fopen(name, "wt")) == NULL) {
    fprintf(stderr, "Unable to create outputfile in get_outputfile()\n");
    exit(1);
  }

  return(temp);
}


char *bitvector_to_string(int number, char *words[])
{
  int highnum = 0, cnt = 1, index = 0;

  for(*buf = 0; number > 0; index++, cnt <<= 1) {
    if (*words[index] == '\n')
      highnum = 1;
    if (number & cnt) {
      if (*buf)
	strcat(buf, "|");
      if (!highnum && *words[index] && str_cmp(words[index], "!UNUSED!"))
	strcat(buf, words[index]);
      else {
	printf("Warning: Unknown bit value \"%d\"\n", cnt);
	sprintf(buf + strlen(buf), "BIT_%d", cnt);
      }
      number -= cnt;
    }
  }
  if (!(*buf))
    strcpy(buf, "0");
  return(buf);
}


char *select_to_string(int number, char *words[])
{
  int index;

  for(index = 0; *words[index] != '\n'; index++)
    if (index == number)
      return(words[index]);
  sprintf(buf1, "SELECT_%d", number);
  return(buf1);
}


int find_index(char *s, char *words[])
{
  int index;

  for(index = 0; *words[index] != '\n'; index++)
    if (!str_cmp(words[index], s))
      return(1 << index);

  return(-1);
}

 
long string_to_bitvector(char *s, char *words[])
{
  char dup[MAX_STRING_LENGTH], *ptr;
  long num = 0, temp;

  if (!str_cmp(s, "0"))
    return(0);

  strcpy(dup, s);
  ptr = strtok(dup, "|");
  while(ptr) {
    if ((temp = find_index(ptr, words)) < 0) {
      if (!strn_cmp(ptr, "BIT_", 4))
	temp = atol(ptr + 4);
      printf("Warning: Unknown Flag \"%s\"\n", ptr);
    }
    if (temp > -1)
      num |= temp;
    ptr = strtok(NULL, "|");
  }
  return(num);
}


long string_to_select(char *s, char *words[])
{
  int index;

  for(index = 0; *words[index] != '\n'; index++)
    if (!str_cmp(words[index], s))
      return(index);
  if (!strn_cmp(s, "SELECT_", 7))
    index = atol(s + 7);
  else
    index = 0;
  printf("Warning: Unknown Selection \"%s\"\n", s);
  return(0);
}


/* returns: 0 if equal, 1 if arg1 > arg2, -1 if arg1 < arg2  */
/* scan 'till found different or end of both                 */
int str_cmp(char *arg1, char *arg2)
{
  int chk, i;

  for (i = 0; *(arg1 + i) || *(arg2 + i); i++)
    if ((chk = tolower(*(arg1 + i)) - tolower(*(arg2 + i))))
      if (chk < 0)
        return (-1);
      else
        return (1);
  return (0);
}



/* returns: 0 if equal, 1 if arg1 > arg2, -1 if arg1 < arg2  */
/* scan 'till found different, end of both, or n reached     */
int strn_cmp(char *arg1, char *arg2, int n)
{
  int chk, i;

  for (i = 0; (*(arg1 + i) || *(arg2 + i)) && (n > 0); i++, n--)
    if ((chk = tolower(*(arg1 + i)) - tolower(*(arg2 + i))))
      if (chk < 0)
        return (-1);
      else
        return (1);

  return (0);
}

